diff --git a/ntirpc/rpc/svc.h b/ntirpc/rpc/svc.h
index 35a4afc..7e242e3 100644
--- a/ntirpc/rpc/svc.h
+++ b/ntirpc/rpc/svc.h
@@ -279,6 +279,9 @@ typedef struct rpc_svcxprt {
 
 	uint16_t xp_flags;	/* flags */
 
+	/* pending output for this xprt */
+	struct poolq_head ioq;
+
 	/*
 	 * union of event processor types
 	 */
diff --git a/src/rpc_dplx_internal.h b/src/rpc_dplx_internal.h
index e797912..f29a670 100644
--- a/src/rpc_dplx_internal.h
+++ b/src/rpc_dplx_internal.h
@@ -77,6 +77,7 @@ static inline void
 rpc_dplx_rec_init(struct rpc_dplx_rec *rec)
 {
 	rpc_dplx_lock_init(&rec->recv.lock);
+	poolq_head_setup(&rec->xprt.ioq);
 }
 
 static inline void
diff --git a/src/svc.c b/src/svc.c
index 9ca5b79..8f67736 100644
--- a/src/svc.c
+++ b/src/svc.c
@@ -153,8 +153,6 @@ svc_init(svc_init_params *params)
 	__svc_params->xprt_u.vc.nconns = 0;
 	mutex_init(&__svc_params->xprt_u.vc.mtx, NULL);
 
-	svc_ioq_init();
-
 #if defined(HAVE_BLKIN)
 	if (params->flags & SVC_INIT_BLKIN) {
 		int r = blkin_init();
diff --git a/src/svc_ioq.c b/src/svc_ioq.c
index 4c799d9..1473731 100644
--- a/src/svc_ioq.c
+++ b/src/svc_ioq.c
@@ -66,40 +66,6 @@
 #include <misc/opr.h>
 #include "svc_ioq.h"
 
-/* Queue per interface, determined per transport socket.
- *
- * Ideally, these would be some variant of weighted fair queuing.  Currently,
- * assuming supplied by underlying OS.
- *
- * The assigned thread should have affinity for the interface.  Therefore, the
- * first thread arriving for each interface is used for all subsequent work,
- * until the interface is idle.  This assumes that the output interface is
- * closely associated with the input interface.
- *
- * Note that this is a static fixed size list of interfaces.  In most cases,
- * many of these entries will be unused.
- *
- * For efficiency, a mask is applied to the ifindex, possibly causing overlap of
- * multiple interfaces.  The size is selected to be larger than expected number
- * of concurrently active interfaces.  Size must be a power of 2 for mask.
- */
-#define IOQ_IF_SIZE (16)
-#define IOQ_IF_MASK (IOQ_IF_SIZE - 1)
-struct poolq_head ioq_ifqh[IOQ_IF_SIZE];
-
-void
-svc_ioq_init(void)
-{
-	struct poolq_head *ifph = &ioq_ifqh[0];
-	int i = 0;
-
-	for (; i < IOQ_IF_SIZE; ifph++, i++) {
-		ifph->qcount = 0;
-		TAILQ_INIT(&ifph->qh);
-		mutex_init(&ifph->qmutex, NULL);
-	}
-}
-
 static inline void
 cfconn_set_dead(SVCXPRT *xprt)
 {
@@ -248,7 +214,7 @@ svc_ioq_write_callback(struct work_pool_entry *wpe)
 {
 	struct xdr_ioq *xioq = opr_containerof(wpe, struct xdr_ioq, ioq_wpe);
 	SVCXPRT *xprt = (SVCXPRT *)xioq->xdrs[0].x_lib[1];
-	struct poolq_head *ifph = &ioq_ifqh[xprt->xp_ifindex & IOQ_IF_MASK];
+	struct poolq_head *ifph = &xprt->ioq;
 
 	svc_ioq_write(xprt, xioq, ifph);
 }
@@ -256,7 +222,7 @@ svc_ioq_write_callback(struct work_pool_entry *wpe)
 void
 svc_ioq_write_now(SVCXPRT *xprt, struct xdr_ioq *xioq)
 {
-	struct poolq_head *ifph = &ioq_ifqh[xprt->xp_ifindex & IOQ_IF_MASK];
+	struct poolq_head *ifph = &xprt->ioq;
 
 	SVC_REF(xprt, SVC_REF_FLAG_NONE);
 	mutex_lock(&ifph->qmutex);
@@ -285,7 +251,7 @@ svc_ioq_write_now(SVCXPRT *xprt, struct xdr_ioq *xioq)
 void
 svc_ioq_write_submit(SVCXPRT *xprt, struct xdr_ioq *xioq)
 {
-	struct poolq_head *ifph = &ioq_ifqh[xprt->xp_ifindex & IOQ_IF_MASK];
+	struct poolq_head *ifph = &xprt->ioq;
 
 	SVC_REF(xprt, SVC_REF_FLAG_NONE);
 	mutex_lock(&ifph->qmutex);
diff --git a/src/svc_ioq.h b/src/svc_ioq.h
index d36d4c5..550d3f7 100644
--- a/src/svc_ioq.h
+++ b/src/svc_ioq.h
@@ -29,7 +29,6 @@
 #include <rpc/svc.h>
 #include <rpc/xdr_ioq.h>
 
-void svc_ioq_init(void);
 void svc_ioq_write_now(SVCXPRT *, struct xdr_ioq *);
 void svc_ioq_write_submit(SVCXPRT *, struct xdr_ioq *);
 
diff --git a/src/svc_rdma.c b/src/svc_rdma.c
index ce16a9b..7903948 100644
--- a/src/svc_rdma.c
+++ b/src/svc_rdma.c
@@ -125,6 +125,8 @@ svc_rdma_ncreate(void *arg, const u_int sendsize, const u_int recvsize,
 		goto freedata;
 	}
 
+	poolq_head_setup(&xprt.ioq);
+
 	return (&xprt->xprt);
 
 freedata:
